#if !defined(AFX_OPENGL_H__4992545F_3CE7_4EF6_A5E6_E4E178DDF8EF__INCLUDED_)
#define AFX_OPENGL_H__4992545F_3CE7_4EF6_A5E6_E4E178DDF8EF__INCLUDED_

#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define M_PI 3.14159265358979323846				// Pi
#define MAX(a,b) ((a) > (b))? (a) : (b)			// Makro zur Maximumberechnung

// OpenGL.h : Header-Datei
//

///////////////////////////////////////////
// Listendefinitionen
// Hauptlisten
#define SZENE							1		// gesamte Szene
#define ATTRIBS							2		// Szenen-Attribute

// Unterlisten
#define KOORDINATENSYSTEM				11		// Koordinatensystem
#define GRUNDSTUECK						12		// Grundstck
	#define GRUNDSTUECK_TEIL			121		// Teilstck
#define ROBOTER							13		// Roboter komplett
	#define KOPF						131		// Kopf des Roboters
	#define ROBORUMPF					132		// Rumpf des Roboters
		#define ARM_RECHTS				1321	// rechter Arm
			#define ARMGELENK			13211	// Armgelenk
			#define ARM1				13212	// Arm 1. Variante
			#define ARMGELENK2			13213	// Unterarmgelenk
			#define UNTERARM			13214	// Unterarm
			#define HAND				13215	// Hand
				#define HANDFLAECHE		132151  // Handflche
				#define FINGER			132152	// "Finger"
		#define ARM_LINKS				1322	// linker Arm
			#define ARM2				13221	// Arm 2. Variante
			#define STRAHLER			13222	// Lichtstrahler
	#define FAHRGESTELL					133		// Fahrgestell des Roboters
		#define RAD						1331	// ein Rad
			#define RADTEIL1			13311	// Felge
			#define RADTEIL2			13312	// Reifen
#define LATERNE							14		// Laterne
	#define MAST						141		// Mast der Laterne
	#define HALTERUNG					142		// Lampenhalterung
	#define LEUCHTER					143		// Lampe
	#define SCHIRM						144		// Lampenschirm
#define LEINWAND						15		// Leinwand
	#define LEINWAND_TEIL				151		// Teilstck der Leinwand
	#define LEINWANDMAST				152		// Mast der Leinwand
		#define LEINWANDMAST_TEIL		1521	// Teil des Mastes
#define BEAMER							16		// Beamer
	#define BEAMER_GEHAEUSE				161		// Gehuse
	#define	BEAMER_LINSE				162		// Linse
	#define BEAMER_HALTERUNG			163		// Halterung fr den Beamer
	#define BEAMER_MAST					164 	// Mast an denen die Halterung fr den Beamer hngt
		#define BEAMER_MAST_TEIL		1641	// Teil des Mastes
	#define	BEAMER_BILD					165		// Linse

///////////////////////////////////////////

typedef struct dVektorTAG // C-Struktur zur Vektornutzung
{
   GLdouble x;    //1. x-Komponente des Vektors
   GLdouble y;    //2. y-Komponente des Vektors
   GLdouble z;    //3. z-Komponente des Vektors

} dVektor;

// Strukturdef. fr COpenGL-Klasse
typedef struct OpenGLStatusTAG
{
	// Globaler Status
	GLboolean bPerspective;						// Perspektive
	GLboolean bInitOGL;							// Initialisierung OpenGL
	GLboolean bLightMode;						// Beleuchtungsmodus
	GLboolean bBlend;							// Tranzparenzeffekte
	GLboolean bZbuffer;							// Z-Buffer
	GLboolean bFillModeLines;					// Fllmodus LINES
	GLboolean bShadingSmooth;					// Smooth- oder Flatshading
	GLboolean bLight;							// Beleuchtungsmodus
	GLboolean bFog;								// Nebel
	GLboolean bTexture;							// Texturen

	// Beleuchtung
	GLboolean bLight0;							// Parallelstrahler
	GLboolean bLight1;							// Laterne
	GLboolean bLight2;							// Robo Strahler
	GLboolean bBeamer;							// Beamer

	// Elemente zeichnen
	GLboolean bKos;								// Koordinatensystem zeichnen

	// sonstiges
	dVektor vLastNormale;						// zuletzt berechnete Normale

} OpenGLStatus;

// Kameraposition
typedef struct OpenGLKameraTAG
{
 //Animationsbeschreibungselemente
 dVektor   vKameraViewPoint;	//Kamera Blickpunkt
 //dVektor   vCurKameraPos;		//akt. KameraPosition

 dVektor	rotate;				// Rotationsvektor;

} OpenGLKamera;

//! Struktur fuer Bearbeitung der Mausevents
typedef struct MouseStructTAG 
{
	int LastState;
	int OldX;
	int OldY;
	int CurX;
	int CurY;
	int ScreenX;
	int ScreenY;
	int MoveX;
	int MoveY;
	double Radius;
	bool ButtonL;
	bool ButtonR;
	float Zoom;
} globMouse;

typedef struct MoveRoboTAG
{
	// Flags zur Bewegung
	bool MoveLeft;							// Flag fr Bewegung nach Links
	bool MoveRight;							// Flag fr Bewegung nach Rechts
	bool MoveForward;						// Flag fr Bewegung nach vorn
	bool MoveBackward;						// Flag fr Bewegung nach hinten

	// Bewegungswerte
	dVektor RoboStandort;					// Standort des Roboters
	dVektor RoboStandortNeu;				// neuer Standort des Roboters
	GLdouble RoboDrehwinkel;				// Ausrichtung des Roboters
	GLdouble ArmLinks;						// Drehwinkel linker Arm
	GLdouble ArmRechts;						// Drehwinkel rechter Arm
	GLdouble Unterarm;						// Drehwinkel rechter Unterarm
	GLdouble UnterarmLinks;					// Drehwinkel linker Unterarm

	GLdouble WheelLeft;						// Linkes Rad Drehwinkel
	GLdouble WheelRight;					// Rechtes Rad Drehwinkel

	GLdouble Kopf;							// Drehwinkel Kopf 

	GLdouble Hand;							// Drehwinkel der Hand
	GLdouble Finger;						// wie weit sind die "Finger" zusammen

	GLfloat SpotCutOff;						// CutOff des Robo-Leuchters
	GLfloat SpotExponent;					// Exponent des Robo-Leuchters
} moveRobo;

typedef struct fRGBAColorTAG				// Typedefinition fr Farbvektor
{
	GLfloat Red;							// Rot-Anteil
	GLfloat Green;							// Grn-Anteil
	GLfloat Blue;							// Blau-Anteil
	GLfloat Alpha;							// Alpha-Wert
} fRGBAColor;

typedef struct fclippingTAG				// Typedefinition fr Clipplanes
{
	GLfloat left;						
	GLfloat right;						
	GLfloat up;							
	GLfloat down;						
	GLfloat front;
	GLfloat back;

} fclipping;

typedef struct fFarbenTAG					// Definition der Farben
{
	fRGBAColor WEISS;						// weiss
	fRGBAColor SCHWARZ;						// schwarz
	fRGBAColor ROT;							// rot
	fRGBAColor GRUEN;						// grn
	fRGBAColor BLAU;						// blau
	fRGBAColor ORANGE;
	fRGBAColor GELB;
	fRGBAColor CYAN;
	fRGBAColor MAGENTA;
	fRGBAColor BRAUN;
	fRGBAColor LILA;
	fRGBAColor OLIVGRUEN;
	fRGBAColor DUNKELGRUEN;
	fRGBAColor MARINEBLAU;
	fRGBAColor TUERKIS;
	fRGBAColor BACKSTEINROT;
	fRGBAColor GRAU;
	fRGBAColor SILBER;
} fFarben;

typedef struct iTexNamenTAG
{
	GLuint Rasen;
	GLuint BeamerBild;

	GLuint RoboKoerper;
	GLuint RoboKopf;
} iTexNamen;

/////////////////////////////////////////////////////////////////////////////
// Rahmen COpenGL 

class COpenGL : public CFrameWnd
{
	DECLARE_DYNCREATE(COpenGL)
public:
	COpenGL();           // Dynamische Erstellung verwendet geschtzten Konstruktor

// Attribute
public:
	fFarben m_fStdFarben;					// Standard-Farbwerte global verwalten

// Operationen
public:
	bool DrawScene();
	bool SetAttribs();
	bool LoadPrepareCurTexture(CString Filename, GLuint *pTexName, 
									GLint MinFilter, GLint MagFilter);
	bool SetMatCol(fRGBAColor *pCurMatColor, GLenum MatFace, GLfloat Ca, GLfloat Cd, GLfloat Cs);
	bool CalcNormale(dVektor *V1, dVektor *V2, dVektor *V3);
	globMouse m_structGlobMouse;
	bool SetCurModel();
	bool SetCurPerspektive();
	OpenGLStatus m_structOGLStatus;
	OpenGLKamera m_structOGLKamera;
	bool OpenGLSzene();
	bool InitOpenGL();
	moveRobo m_structMoveRobo;

// berschreibungen
	// Vom Klassen-Assistenten generierte virtuelle Funktionsberschreibungen
	//{{AFX_VIRTUAL(COpenGL)
	//}}AFX_VIRTUAL

// Implementierung
protected:
	fclipping m_structClipplanes;
	iTexNamen m_TexturNamen;
	bool SetRenderingContext();
	virtual ~COpenGL();

	// Roboter
//	GLUquadricObj* m_pRadPart1;
	GLUquadricObj* m_pRadPart2;
	GLUquadricObj* m_pRumpf;
	GLUquadricObj* m_pKopf;
	GLUquadricObj* m_pArmgelenk;
	GLUquadricObj* m_pArmgelenk2;

	// Laterne
	GLUquadricObj* m_pMast;
	GLUquadricObj* m_pHalterung;
	GLUquadricObj* m_pLeuchter;
	GLUquadricObj* m_pBeamer;
//	GLUquadricObj* m_pSchirm;

	// Generierte Nachrichtenzuordnungsfunktionen
	//{{AFX_MSG(COpenGL)
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ fgt unmittelbar vor der vorhergehenden Zeile zustzliche Deklarationen ein.

#endif // AFX_OPENGL_H__4992545F_3CE7_4EF6_A5E6_E4E178DDF8EF__INCLUDED_
